<?php
namespace Controllers;
// Контроллер для товаров
class Products extends Controller {

	public function __construct() {
		parent::__construct();
		$this->model=new \Models\Products($this->db, $this->f3);
	}

	// Поиск по штрих-коду
	function search_barcodes() {
		$term=$this->f3->get('GET.product_search');
		$count=$this->f3->get('GET.coeff');
		$not_add=$this->f3->get('GET.not_add');
		if(!$count)$count=1;
		if(!$term) {
			$this->f3->set('page_data', $this->messages->message('not_enough_data'));
			return;
		}

		$page=$this->f3->get('GET.page');
		if(!$page) $page=1;

		$s=new \Models\Settings($this->f3, $this->db);
		$sets=$s->get_settings(array('weight_code_pr', 'card_code_pr'));
		// Проверяем штрих-код на принадлежность к весовому типу
		if(mb_substr($term, 0, 2)==$sets['weight_code_pr'] && mb_strlen($term)==13) {
			$barcode=$term;
			$term=mb_substr($barcode, 0, 7);
			$count=mb_substr($barcode, 7, 5);
			$count=number_format(round($count/1000, 3), 3, '.', '');
		}

		if(!$not_add) {
			// Проверяем штрих-код на принадлежность к карте клиента
			if(mb_substr($term, 0, 2)==$sets['card_code_pr'] && mb_strlen($term)==13) {
				$this->f3->reroute('/partners/search?card='.$term);
				return;
			}
		}

		$limit=$this->f3->get('lines_of_page');
		$res=$this->model->search($term, ($page-1)*$limit, $limit, false);
		if(count($res['products'])>1 || $not_add==1) {
			$this->f3->set('pagination', $this->pagination($res['count'], $page));
			$this->f3->set('products', $res['products']);
			$this->f3->set('page_data', $this->model->page_data('search'));
		}
		elseif(count($res['products'])==1) {
			$res=array(
				'success'=>true,
				'txt'=>$this->messages->message('loading')['txt'],
				'next_step'=>'/receipts/add_line?product_id='.$res['products'][0]['id'].'&count='.$count
			);
			echo json_encode($res);
			exit();
		}
		else {
			$this->f3->set('page_data', $this->messages->message('not_found'));
		}
	}

	// Отображение быстрых товаров
	function fast_products() {
		$page=$this->f3->get('GET.page');
		if(!$page) $page=1;
		$limit=$this->f3->get('lines_of_page');
		$res=$this->model->fast_products(($page-1)*$limit, $limit);
		if(count($res['products'])>0) {
			$this->f3->set('pagination', $this->pagination($res['count'], $page));
			$this->f3->set('products', $res['products']);
			$this->f3->set('page_data', $this->model->page_data('fast_products'));
		}
		else {
			$this->f3->set('page_data', $this->messages->message('not_found'));
		}
	}

	// Показ картинки товара
	function img() {
		$img=$this->f3->get('GET.img');
		if(!$img) {
			$this->f3->set('page_data', $this->messages->message('not_found'));
		}
		else {
			$api_data=$this->controller->get_api_config();
			$subdomen=$api_data['domen'];
			$subdomen=mb_substr($subdomen, mb_strpos($subdomen, '//')+2);
			$subdomen=mb_substr($subdomen, 0, mb_strpos($subdomen, '.'));
			$this->f3->set('img', $api_data['domen'].'/upload/'.$subdomen.'/300_300/'.$img);
			$this->f3->set('page_data', $this->model->page_data('img'));
		}
	}

	// Узнаем остатки
	function currentstock() {
		$line_id=$this->f3->get('GET.line_id');
		if(!$line_id) {
			$this->f3->set('page_data', $this->model->page_data('search_form'));
		}
		else {
			// Узнаем id продукта
			$l=new \Models\Receiptlines($this->db, $this->f3);
			$line=$l->select_lines(array('id=?', $line_id), array('limit'=>1));
			if(empty($line)) {
				$this->f3->set('page_data', $this->messages->message('not_found'));
			}
			else {
				$prod=$this->model->select_product(array('p.id=?', $line[0]['product_id']), array('limit'=>1));
				if(empty($prod)) {
					$this->f3->set('page_data', $this->messages->message('not_found'));
				}
				else {
					$this->f3->set('product', $prod[0]);
					$this->f3->set('page_data', $this->model->page_data('currentstock'));
				}
			}
		}
	}

	// Список категорий и товаров
	function list() {
		$c=new \Models\Productcategories($this->db, $this->f3);
		$category_id=$this->f3->get('GET.category_id');
		$parent=array();
		if(!$category_id) {
			$category_id=0;
		}
		else {
			// Узнаем название родительской категории, если она есть
			$parent=$c->select_productcategories(array('id=?', $category_id), array('limit'=>1));
		}
		$this->f3->set('parent', $parent[0]);
		$page=$this->f3->get('GET.page');
		if(!$page) $page=1;
		$this->f3->set('page', $page);
		$lines_of_list=$this->f3->get('lines_of_list');

		// Получаем категории
		$categories=$c->select_productcategories(array('parent_id=?', $category_id), array('order'=>'title', 'offset'=>($page-1)*$lines_of_list, 'limit'=>$lines_of_list));
		$this->f3->set('categories', $categories);
		$products=array();
		// сколько всего категорий
		$cat_all=$c->count(array('parent_id=?', $category_id));
		// Узнаем надо ли загрузить товары, чтобы добить число позиций на странице до 10
		$cat_count=count($categories);
		if($cat_count<$lines_of_list) {
			// Узнаем позицию начала товаров
			// Начало выборки
			$cat_page=intdiv($cat_all, $lines_of_list); // Сколько страниц полностью занято категориями
			$cat_last_page=$cat_all%$lines_of_list; // Сколько категорий на последней странице с категориями

			$prod_page=$page-$cat_page;// Сколько страниц было с товарами
			$prod_first_page=$lines_of_list-$cat_last_page; // Сколько товаров было на первой странице с товарами
			// Считаем начало выборки товаров
			if($prod_page==1) {// Если это первая странице где появляются товары
				$offset=0;
				$limit=$lines_of_list-$cat_last_page;
			}
			else {
				$offset=($prod_page-2)*$lines_of_list+$prod_first_page;
				$limit=$lines_of_list;
			}
			$products=$this->model->select_product(array('category_id=?', $category_id), array('order'=>'title', 'limit'=>$offset.', '.$limit));
		}
		$this->f3->set('products', $products);
		// Узнаем сколько всего товаров
		$prods_all=$this->model->count(array('category_id=? AND date_delete=0', $category_id));
		// Кол-во страниц всего
		$page_all=ceil(($cat_all+$prods_all)/$lines_of_list); //Сколько всего должно быть страниц
		$this->f3->set('page_all', $page_all);
		$this->f3->set('page_data', $this->model->page_data('list'));
	}
}
