<?php
namespace Models;

// Модель товаров
class Products extends \DB\SQL\Mapper {
	public $db, $f3;

	public function __construct($db, $f3) {
		parent::__construct($db, 'products' ); // подключаемся к таблице товаров
		$this->db=$db;
		$this->f3=$f3;
	}

	// Информация об открываемой странице
	function page_data($page='sales') {
		$pages=array(
			'search_form'=>array(
				'title'=>'Поиск товара',
				'inc'=>'products_search_form.htm'
			),
			'search'=>array(
				'title'=>'Результаты поиска',
				'inc'=>'products_search.htm'
			),
			'fast_products'=>array(
				'title'=>'Быстрые товары',
				'inc'=>'products_search.htm'
			),
			'img'=>array(
				'title'=>'Изображение товара',
				'inc'=>'products_img.htm'
			),
			'currentstock'=>array(
				'title'=>'Остатки на складе',
				'inc'=>'products_currentstock.htm'
			),
			'list'=>array(
				'title'=>'Категории и товары',
				'inc'=>'products_list.htm'
			)
		);
		$pages[$page]['page']=$page;
		return $pages[$page];
	}

	// Выбор из базы
	function select_product($query=null, $options=null) {
		if(!$query) {
			$query=array('p.id>?', 0); // чтобы хоть что-то вставить в фильтры
		}

		// формируем сортировку и лимиты
		$option='';
		if($options['order']) {
			$option.=' ORDER BY '.$options['order'];
		}
		if($options['limit']) {
			$option.=' LIMIT '.$options['limit'];
		}
		// собственно сам запрос
		$sql="SELECT p.*, c.title AS `category_title` FROM `products` AS `p` LEFT JOIN `productcategories` AS `c` ON c.id=p.category_id WHERE p.date_delete=0 AND ".$query[0].' '.$option;
		$res=$this->db->exec($sql, $query[1]);
		return $res;
	}

	// Составление релевантного запроса
	function relevant($search0) {
		$search0=str_replace("'","\'",$search0);
		$search0=trim(htmlspecialchars($search0));
		$search=str_replace("'",'',$search0);
		$search=str_replace('"','',$search);
		// Разобрать искомую строку $search на отдельные слова
		preg_match_all('/[[:alnum:]]{1,}/isu', stripslashes($search), $matches);
		$words = array_unique($matches[0]);
 
		$true_words=Array();
		if (count($words)) {
    		foreach($words as $word) {
        		// Обрабатывать только слова длиннее 1 символов
        		if (strlen($word)>1) {
            		// От слов длиннее 7 символов отрезать 2 последних буквы
            		if (strlen($word)>7) {
                		$word=substr($word,0,(strlen($word)-2));
            		}
            		// От слов длиннее 5 символов отрезать последнюю букву
            		elseif (strlen($word)>5) {
                		$word=substr($word,0,(strlen($word)-1));
            		}
            		$true_words[]=addcslashes(addslashes($word),'%_');
        		}
    		}    
		}
		// Список уникальных поисковых слов
		$true_words=array_unique($true_words);
		if(count($true_words)==0) {
			return array("query"=>"", "where"=>"", "order"=>"title");
		}
		$coeff_title=round((20/count($true_words)),2);
 
		// Формируем запрос к базе
		// Условия для полного совпадения фразы в заголовке и тексте
		$query = ", (CASE WHEN p.title LIKE '%".$search."%' THEN 60 ELSE 0 END ";
		// Условия для каждого из слов
		foreach($true_words as $word) {
    		$query .= "+ CASE WHEN p.title LIKE '%".$word."%' THEN ".number_format($coeff_title, 2, '.', '')." ELSE 0 END ";
		}
		$query.=") AS relevant ";
 
		// Условие выборки - вхождение фразы или отдельных слов в заголовок или текст
		$where= " OR (";
		$where .= " p.title LIKE '%".$search0."%'";
		// Условия для каждого из слов
		foreach($true_words as $word) {
    		$where .= " OR p.title LIKE '%".$word."%'";
		}
		$where.=") ";
		return array("query"=>$query, "where"=>$where, "order"=>"relevant DESC");
	}

	// Быстрые товары
	function fast_products($start, $limit) {
		$count=0;
		$products=array();
		$limit_txt='';
		if($start>0 && $limit>0) {
			$limit_txt='LIMIT '.$start.', '.$limit;
		}
		elseif($limit>0) {
			$limit_txt='LIMIT '.$limit;
		}
		$sql="SELECT p.*, c.title AS category_title FROM products AS p LEFT JOIN productcategories AS c ON c.id=p.category_id WHERE p.fast>0 AND p.date_delete=0 ORDER BY title ".$limit_txt;
		$products=$this->db->exec($sql);
		$count=count($products);
		if($count>0) {
			foreach ($products as $i => $p) {
				$products[$i]['img']=json_decode($p['img'], true);
			}
		}
		if($limit_txt) {
			$sql="SELECT COUNT(*) AS count FROM products WHERE date_delete=0 AND fast>0";
			$res=$this->db->exec($sql);
			$count=$res[0]['count'];
		}
		return array(
			'success'=>true,
			'count'=>$count,
			'products'=>$products
		);
	}

	// поиск товара
	function search($term, $start=0, $limit=30, $barcodes_only=false) {
		$term=mb_strtoupper($term);
		$limit_txt='';
		if($start>0 && $limit>0) {
			$limit_txt='LIMIT '.$start.', '.$limit;
		}
		elseif($limit>0) {
			$limit_txt='LIMIT '.$limit;
		}
		// Пытаемся найти штрих-код
		$bsql="SELECT p.*, c.title AS category_title FROM barcodes AS b LEFT JOIN products AS p ON p.id=b.product_id AND p.date_delete=0 LEFT JOIN productcategories AS c ON c.id=p.category_id WHERE b.barcode=?";
		$products=$this->db->exec($bsql, $term);
		$count=1;
		if(empty($products) && !$barcodes_only) { // если не находим то ищем по названию и коду
			$w=$this->relevant($term);
			$sql="SELECT p.*, c.title AS category_title ".$w['query']." FROM products AS p LEFT JOIN productcategories AS c ON c.id=p.category_id WHERE (p.date_delete=0 AND (p.code=? ".$w['where'].")) ORDER BY ".$w['order'].' '.$limit_txt;
			$products=$this->db->exec($sql);
			$count=count($products);
			if($limit_txt) {
				$sql="SELECT COUNT(*) AS count ".$w['query']." FROM products AS p WHERE (p.date_delete=0 AND (p.code=? ".$w['where']."))";
				$res=$this->db->exec($sql, $term);
				$count=$res[0]['count'];
			}
		}
		if($count>0) {
			foreach ($products as $i => $p) {
				$products[$i]['img']=json_decode($p['img'], true);
			}
		}
		return array(
			'success'=>true,
			'count'=>$count,
			'products'=>$products
		);
	}

	// Сохранение товаров
	function save_product($data, $action='add') {
		if(!isset($data['title'])) {
			return array(
				'success'=>false,
				'txt'=>'not_enough_data'
			);
		}

		$this->reset();
		if($action=='change' && $data['id']) {
			$this->load(array('id=?', $data['id']));
		}

		$data['title_lg']=mb_strtoupper($data['title']);

		$this->copyFrom($data);
		$this->save();
		if($this->dry()) {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
		$this->reset();
		return array(
			'success'=>true,
			'txt'=>'save_successful'
		);
	}
}
