<?php
namespace Models;

// Модель строк чека
class Receiptlines extends \DB\SQL\Mapper {
	public $db, $f3;

	public function __construct($db, $f3) {
		parent::__construct($db, 'receiptlines' ); // подключаемся к таблице строк чека
		$this->db=$db;
		$this->f3=$f3;
	}

	// Выбираем строки из базы
	function select_lines($query, $option=null) {
		$lines=array();
		$this->load($query, $option);
		if(!$this->dry()) {
			do {
				$lines[]=$this->cast();
			}
			while($this->skip());
		}
		$this->reset();
		return $lines;
	}

	// Добавляем строку в чек
	function add_line($receipt_id, $line, $count=1) {
		$line['receipt_id']=$receipt_id;
		$line['count']=$count;
		$line['product_id']=$line['id'];
		$line['product_title']=$line['title'];
		if(!isset($line['price0'])) {
			$line['price0']=$line['price'];
		}
		unset($line['id']);
		$this->reset();
		$this->copyFrom($line);
		if($this->insert()) {
			$id=$this->get('_id');
			return array(
				'success'=>true,
				'txt'=>'save_successful',
				'line'=>$line,
				'reload'=>'.receiptlines',
				'id'=>$id,
				'scroll'=>'.receiptlines_div',
				'scrollto'=>'#line_'.$id,
				'close_all_window'=>true
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
	}

	// Меняем строку в чеке
	function change_line($line_id, $count, $price, $discount=0) {
		$this->reset();
		$this->load(array('id=?', $line_id), array('limit'=>1));
		if($this->dry()) {
			return array(
				'success'=>false,
				'txt'=>'not_found'
			);
		}
		$line=$this->cast();
		if($count) {
			$line['count']=$count;
		}
		if($price) {
			if($line['vat_type']>0) {
				$price-=$price/(100+$line['vat'])*$line['vat'];
			}
			$line['price']=$price;
		}
		$line['line_discount']=$discount;
		if(!$discount) {
			$line['price0']=$price;
		}
		$this->copyFrom($line);
		if($this->update()) {
			return array(
				'success'=>true,
				'txt'=>'save_successful',
				'reload'=>'.receiptlines',
				'scroll'=>'.receiptlines_div',
				'scrollto'=>'#line_'.$line_id,
				'id'=>$line_id,
				'line'=>$line,
				'close_all_window'=>true
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
	}

	// Удаляем строку из чека
	function delete_line($line_id) {
		$this->reset();
		if($line_id=='all') {
			// Узнаем открытый чек
			if($line_id=='all') {
				$c=new \Models\Controller($this->f3);
				$r=new \Models\Receipts($this->db, $this->f3);
				$api_data=$c->get_api_config();
				$receipt=$r->get_open_receipt($api_data['rm_id'], $api_data['store_id'], false);
				if($receipt['success']) {
					$res=$this->db->exec("DELETE FROM `receiptlines` WHERE `receipt_id`=?", $receipt['receipt']['id']);
					if($res===false) {
						return array(
							'success'=>false,
							'txt'=>'db_error'
						);
					}
				}
			}
		}
		else {
			$this->load(array('id=?', $line_id), array('limit'=>1));
			if($this->dry()) {
				return array(
					'success'=>false,
					'txt'=>'not_found'
				);
			}
			$this->erase();
		}

		return array(
			'success'=>true,
			'txt'=>'save_successful',
			'reload'=>'.receiptlines',
			'close_all_window'=>true
		);
	}

	// Узнаем сумму строк
	function get_sum($lines, $discount=0) {
		$sum=0;
		if($lines) {
			foreach ($lines as $l) {
				$s=($l['price']+($l['price']*(($l['vat']/100)*max(0, $l['vat_type']))));
				if($l['discount']) {
					$s-=$s*($l['discount']/100);
				}
				$sum+=$s*$l['count'];
			}
		}
		if($discount) {
			$sum-=$sum*($discount/100);
		}
		$sum=round($sum, 2);
		return $sum;
	}

	// Сохраняем строки в базе данных
	function save_lines($receipt_id, $discount=0) {
		if(!$receipt_id) {
			return array(
				'success'=>false,
				'txt'=>'not_enough_data'
			);
		}

		$res=$this->db->exec("UPDATE `receiptlines` SET `discount`=? WHERE `receipt_id`=?", array($discount, $receipt_id));
		if($res===false) {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
		return array(
			'success'=>true,
			'txt'=>'save_successful',
			'reload'=>'.receiptlines',
			'reload_menu'=>true,
			'close_all_window'=>true
		);
	}
}
